@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2013 Yuichi Watanabe
  */
 
 #include <core/assert.h>
@@ -38,7 +38,6 @@
 #include <core/printf.h>
 #include <core/string.h>
 #include <io/pci.h>
-#include "apic_pass.h"
 #include "callrealmode.h"
 #include "convert.h"
 #include "current.h"
@@ -168,52 +167,50 @@ vmctl_init (void)
 }
 
 static void
-initregs (bool vbsp, u8 bios_boot_drive)
+load_guestboot(u8 bios_boot_drive)
 {
 	void *p;
 	u8 boot_drive;
 
-	vcpu_reset();
+	if (!cpu_is_bsp()) {
+		return;
+	}
 
-	if (cpu_is_bsp()) {
-		/*
-		 * vm0 boots from host BOIS.
-		 */
-		p = mapmem_hphys (GUEST_BOOT_OFFSET, GUEST_BOOT_LENGTH,
-				  MAPMEM_WRITE);
-		if (p == NULL) {
-			panic("Failed to map the host BIOS.");
-		}
-		memcpy (p, guest_boot_start, GUEST_BOOT_LENGTH);
-		unmapmem (p, GUEST_BOOT_LENGTH);
-		current->vmctl.write_realmode_seg (SREG_CS, 0x0);
-		current->vmctl.write_ip (GUEST_BOOT_OFFSET);
+	/*
+	 * vm0 boots from host BOIS.
+	 */
+	p = mapmem_hphys (GUEST_BOOT_OFFSET, GUEST_BOOT_LENGTH,
+			  MAPMEM_WRITE);
+	if (p == NULL) {
+		panic("Failed to map the host BIOS.");
+	}
+	memcpy (p, guest_boot_start, GUEST_BOOT_LENGTH);
+	unmapmem (p, GUEST_BOOT_LENGTH);
+	current->vmctl.write_realmode_seg (SREG_CS, 0x0);
+	current->vmctl.write_ip (GUEST_BOOT_OFFSET);
 
-		if (param_get_str(VM0_NAME ".boot_int18", 0, NULL, 0)
-		    == VMMERR_SUCCESS) {
-			current->vmctl.write_general_reg (GENERAL_REG_RDI, 1);
-			printf("Boot using int18.\n");
+	if (param_get_str(VM0_NAME ".boot_int18", 0, NULL, 0)
+	    == VMMERR_SUCCESS) {
+		current->vmctl.write_general_reg (GENERAL_REG_RDI, 1);
+		printf("Boot using int18.\n");
+	} else {
+		if (param_get_u8(VM0_NAME ".boot_drive", 0,
+				 &boot_drive) == VMMERR_SUCCESS) {
+			current->vmctl.write_general_reg(
+				GENERAL_REG_RCX, boot_drive);
+			printf("Boot from 0x%x drive.\n", boot_drive);
 		} else {
-			if (param_get_u8(VM0_NAME ".boot_drive", 0,
-					 &boot_drive) == VMMERR_SUCCESS) {
-				current->vmctl.write_general_reg(
-					GENERAL_REG_RCX, boot_drive);
-				printf("Boot from 0x%x drive.\n", boot_drive);
-			} else {
-				current->vmctl.write_general_reg(
-					GENERAL_REG_RCX, bios_boot_drive);
-				printf("Boot from 0x%x drive.\n",
-				       bios_boot_drive);
-			}
-			if (param_get_str(VM0_NAME ".boot_partition",
-					  0, NULL, 0) == VMMERR_SUCCESS) {
-				current->vmctl.write_general_reg(
-					GENERAL_REG_RSI, 1);
-				printf("Use loader of active partition\n");
-			}
+			current->vmctl.write_general_reg(
+				GENERAL_REG_RCX, bios_boot_drive);
+			printf("Boot from 0x%x drive.\n",
+			       bios_boot_drive);
+		}
+		if (param_get_str(VM0_NAME ".boot_partition",
+				  0, NULL, 0) == VMMERR_SUCCESS) {
+			current->vmctl.write_general_reg(
+				GENERAL_REG_RSI, 1);
+			printf("Use loader of active partition\n");
 		}
-	} else if (!vbsp) {
-		apic_enter_wait_for_sipi_state();
 	}
 }
 
@@ -289,7 +286,8 @@ setup_vm (void)
 	call_initfunc ("passcpu");
 
 	sync_all_processors ();
-	initregs (vbsp, bios_boot_drive);
+	vcpu_reset();
+	load_guestboot(bios_boot_drive);
 	current->initialized = true;
 
 	sync_all_processors ();
