@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2013 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,14 +30,18 @@
 #include <common/common.h>
 #include <core/assert.h>
 #include <core/cpu.h>
+#include <core/initfunc.h>
 #include <core/mm.h>
 #include <core/param.h>
 #include <core/printf.h>
 #include <core/spinlock.h>
 #include <io/pci.h>
+#include "apic_pass.h"
 #include "current.h"
+#include "gbiosloader.h"
 #include "int.h"
 #include "pcpu.h"
+#include <core/sleep.h>
 #include "vm.h"
 
 static size_t driver_data_size = 0;
@@ -55,7 +59,8 @@ vm_alloc_driver_data(size_t size)
 	ASSERT(cpu_is_bsp());
 
 	offset = driver_data_size;
-	driver_data_size += ROUND_UP(size, 8);
+	driver_data_size += ROUND_UP(size, sizeof(u64));
+
 	return offset;
 }
 
@@ -63,7 +68,7 @@ void *
 vm_get_driver_data(drvdata_hdl_t handle)
 {
 	ASSERT(handle < driver_data_size);
-	return current->vm->driver_data + handle;
+	return (void *)((ulong)(current->vm->driver_data) + handle);
 }
 
 unsigned int
@@ -85,6 +90,29 @@ vm_get_name()
 	return current->vm->name;
 }
 
+u32
+vm_get_id(void)
+{
+	char name[VM_NAME_BUF_SIZE];
+	u32 id = 0;
+	int i = 0;
+	vmmerr_t err;
+
+	while ((err = param_get_str("vm", i++, name, VM_NAME_BUF_SIZE))
+	       != VMMERR_NOTEXIST) {
+		if (err != VMMERR_SUCCESS) {
+			continue;
+		}
+		if (strcmp(name, VM0_NAME) != 0) {
+			id++;
+		}
+		if (strcmp(name, vm_get_name()) == 0) {
+			return id;
+		}
+	}
+	return (u32)-1;
+}
+
 apic_id_t
 vm_vbsp_apic_id(void)
 {
@@ -245,13 +273,11 @@ vm_new(void)
 	LIST2_ADD(vm_list, vm_list, vm);
 	spinlock_unlock(&vm_list_lock);
 
-	int_init_hook(vm);
-
 	vm->vbsp_apic_id = get_apic_id();
 	return vm;
 }
 
-struct vm *
+void
 vm_dump(void)
 {
 	struct vm *vm;
@@ -276,5 +302,42 @@ vm_dump(void)
 	}
 	pci_dump_assignment();
 	spinlock_unlock(&vm_dump_lock);
-	return vm;
+	return;
+}
+
+void
+vm_iommu_enabled(void)
+{
+	current->vm->iommu_enabled = true;
+}
+
+vmmerr_t
+vm_reset(void)
+{
+	if (current->vm->gmm.clear_mem == NULL ||
+	    !current->vbsp) {
+		return VMMERR_NOSUP;
+	}
+
+	printf("Reset %s\n", vm_get_name());
+
+	/*
+	 * Initialize other cpus.
+	 */
+	apic_send_init_others();
+
+	usleep(100*1000);
+
+	/*
+	 * Reset vcpu and load guest BIOS.
+	 */
+	printf("Reset cpu 0x%x\n", get_cpu_id());
+	vcpu_reset();
+	vcpu_reset_apic();
+
+	current->vm->gmm.clear_mem();
+	gbios_load();
+	call_initfunc("resetvm");
+
+	return VMMERR_SUCCESS;
 }
