@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2013 Yuichi Watanabe
  */
 
 #include <core/assert.h>
@@ -241,7 +241,7 @@ do_exception (void)
 					    &errc);
 				current->u.vt.intr.exception_errcode = errc;
 				if (vii.s.vector == EXCEPTION_GP) {
-					panic ("DEBUG: General protection fault on CPU%d. error code 0x%lx\n", get_cpu_id(), errc);
+					printf ("DEBUG: General protection fault on CPU%d. error code 0x%lx\n", get_cpu_id(), errc);
 				}
 			}
 			current->u.vt.intr.instruction_len = 0;
@@ -598,14 +598,23 @@ err:
 	panic ("do_task_switch: error %d", r);
 }
 
+/* Copy from bitvisor 1.3 and modify */
+static void
+do_xsetbv (void)
+{
+	cpu_emul_xsetbv();
+	add_ip ();
+}
+
 static void
 vt_exit_reason (void)
 {
 	ulong exit_reason;
 
 	asm_vmread (VMCS_EXIT_REASON, &exit_reason);
-	if (exit_reason & EXIT_REASON_VMENTRY_FAILURE_BIT)
+	if (exit_reason & EXIT_REASON_VMENTRY_FAILURE_BIT) {
 		panic ("VM Entry failure.");
+	}
 	switch (exit_reason & EXIT_REASON_MASK) {
 	case EXIT_REASON_MOV_CR:
 		do_mov_cr ();
@@ -653,14 +662,10 @@ vt_exit_reason (void)
 		do_task_switch ();
 		break;
 	case EXIT_REASON_INIT_SIGNAL:
-		/*
-		 * The logical processor received INIT signal.
-		 * Some controller (For example, ACPI controller) might
-		 * asserted INIT signal.
-		 * Reboot machine from CPU0.
-		 */
 		if (cpu_is_bsp()) {
 			/*
+			 * Some controller (For example, ACPI
+			 * controller) might asserted INIT signal.
 			 * To reboot machine, we seem to need to
 			 * execute vmxoff on CPU0.
 			 */
@@ -668,10 +673,14 @@ vt_exit_reason (void)
 			apic_send_init(get_apic_id());
 			freeze();
 		} else {
+			printf("Reset cpu 0x%x\n", get_cpu_id());
 			vcpu_reset();
-			apic_enter_wait_for_sipi_state();
+			vcpu_reset_apic();
 		}
 		break;
+	case EXIT_REASON_XSETBV:
+		do_xsetbv ();
+		break;
 	case EXIT_REASON_EPT_VIOLATION:
 		vt_ept_violation();
 		break;
@@ -847,6 +856,7 @@ vt_mainloop (void)
 			/* not switching */
 			vt_vm_run ();
 		}
+		cpu_mmu_spt_tlbflush ();
 		vt_event_delivery_check ();
 		vt_exit_reason ();
 		enable_interrupt();
